\name{hist.simpleboot}

\alias{hist.simpleboot}

\title{
  Histograms for bootstrap sampling distributions.
}

\usage{
\method{hist}{simpleboot}(x, do.rug = FALSE, xlab = "Bootstrap samples", main = "", ...)
}

\description{
  Construct a histogram of the bootstrap distribution of univariate
  statistic.
}

\arguments{
  \item{x}{An object of class \code{"simpleboot"} returned from either
    \code{one.boot}, \code{two.boot}, or \code{pairs.boot}.}
  \item{do.rug}{Should a rug of the bootstrap distribution be plotted
    under the histogram?}
  \item{xlab}{The label for the x-axis.}
  \item{main}{The title for the histogram.}
  \item{...}{Other arguments passed to \code{hist}.}
}

\value{
  Nothing is returned.
}

\details{
  \code{hist} constructs a histogram for the bootstrap distribution of a
  univariate statistic.  It cannot be used with linear model or loess
  bootstraps.  In the histogram a red dotted line is plotted denoting
  the observed value of the statistic.
}

\author{Roger D. Peng}

\keyword{hplot}

\examples{
x <- rnorm(100)

## Bootstrap the 75th percentile
b <- one.boot(x, quantile, R = 1000, probs = 0.75)
hist(b)
}

\eof
\name{lm.boot}
\alias{lm.boot}
\alias{print.lm.simpleboot}


\title{
  Linear model bootstrap.
}

\usage{
lm.boot(lm.object, R, rows = TRUE, new.xpts = NULL, ngrid = 100,
        weights = NULL)
}

\description{
  Bootstrapping of linear model fits (using \code{lm}).  Bootstrapping
  can be done by either resampling rows of the original data frame or
  resampling residuals from the original model fit.
}

\arguments{
  \item{lm.object}{A linear model fit, produced by \code{lm}.}
  \item{R}{The number of bootstrap replicates to use.}
  \item{rows}{Should we resample rows?  Setting \code{rows} to
    \code{FALSE} indicates resampling of residuals.}
  \item{new.xpts}{Values at which you wish to make new
    predictions.  If specified, fitted values from each bootstrap sample
    will be stored.}
  \item{ngrid}{If \code{new.xpts} is \code{NULL} and the regression is 2
    dimensional, then predictions are made on an evenly spaced grid
    (containing \code{ngrid} points)
    spanning the range of the predictor values.}
  \item{weights}{Reseampling weights; a vector of length equal to the
    number of observations.}
}

\value{
  An object of class \code{"lm.simpleboot"} (which is a list) containing the
  elements:
  \item{method}{Which method of bootstrapping was used (rows or
    residuals).}
  \item{boot.list}{A list containing values from each of the bootstrap
    samples.  Currently, bootstrapped values are model coefficients,
    residual sum of squares, R-square, and fitted values for predictions.}
  \item{orig.lm}{The original model fit.}
  \item{new.xpts}{The locations where predictions were made.}
  \item{weights}{The resampling weights.  If none were used, this
    component is \code{NULL}}
}

\details{
  Currently, \code{"lm.simpleboot"} objects have a simple \code{print}
  method (which shows the original fit), a \code{summary} method and a
  \code{plot} method.
}


\seealso{
  The \code{plot.lm.simpleboot} method.
}


\author{Roger D. Peng}

\keyword{regression}

\examples{
data(airquality)
attach(airquality)
set.seed(30)
lmodel <- lm(Ozone ~ Wind)
lboot <- lm.boot(lmodel, R = 1000)
summary(lboot)

## With weighting
w <- runif(nrow(model.frame(lmodel)))
lbootw <- lm.boot(lmodel, R = 1000, weights = w)
summary(lbootw)

## Resample residuals
lboot2 <- lm.boot(lmodel, R = 1000, rows = FALSE)
summary(lboot2)
}

\eof
\name{lm.simpleboot.methods}

\alias{summary.lm.simpleboot}
\alias{print.summary.lm.simpleboot}
\alias{fitted.lm.simpleboot}

\title{
  Methods for linear model bootstrap.
}

\usage{
\method{summary}{lm.simpleboot}(object, ...)
\method{print}{summary.lm.simpleboot}(x, ...)
\method{fitted}{lm.simpleboot}(object, ...)
}

\description{
  Methods for \code{"lm.simpleboot"} class objects.  
}

\arguments{
  \item{object}{An object of class \code{"lm.simpleboot"}, as returned by
    \code{lm.boot}.}
  \item{x}{An object of class \code{"summary.lm.simpleboot"}, a result of
    a call to \code{summary.lm.simpleboot}.}
  \item{...}{Other arguments passed to and from other methods.}
}

\details{
  \code{print} is essentially the same as the usual
  printing of a linear model fit, except the bootstrap standard errors
  are printed for each model coefficient.

  \code{fitted} returns the fitted values from each
  bootstrap sample for the predictor values specified by the
  \code{new.xpts} in \code{lm.boot} (or from the grid if \code{new.xpts}
  was not specified).  This is a p x R matrix where p is the number of
  points where prediction was desired and R is the number of bootstrap
  samples specified.  Using \code{fitted} is the equivalent
  of using \code{samples(object, name = "fitted")}.
}

\value{
  \code{summary} returns a list containing the original
  estimated coefficients and their bootstrap standard errors.
}

\seealso{
  \code{lm.boot}.
}

\examples{
data(airquality)
attach(airquality)
lmodel <- lm(Ozone ~ Wind + Solar.R)
lboot <- lm.boot(lmodel, R = 300)
summary(lboot)
}

\keyword{regression}

\author{Roger D. Peng}

\eof
\name{loess.boot}
\alias{loess.boot}
\alias{print.loess.simpleboot}

\title{
  2-D Loess bootstrap.
}

\usage{
loess.boot(lo.object, R, rows = TRUE, new.xpts = NULL, ngrid = 100,
           weights = NULL)
}

\description{
  Bootstrapping of loess fits produced by the \code{loess} function in
  the \code{modreg} package.  Bootstrapping can be done by resampling
  rows from the original data frame or resampling residuals from the
  original model fit.
}

\arguments{
  \item{lo.object}{A loess fit, produced by \code{loess}.}
  \item{R}{The number of bootstrap replicates.}
  \item{rows}{Should we resample rows?  Setting \code{rows} to
    \code{FALSE} indicates resampling of residuals.}
  \item{new.xpts}{Locations where new predictions are to be made.  If
    \code{new.xpts} is \code{NULL}, then an evenly spaced grid spanning
    the range of X (containing \code{ngrid} points) is used.  In either
    case}
  \item{ngrid}{Number of grid points to use if \code{new.xpts} is
    \code{NULL}.}
  \item{weights}{Resampling weights; a vector with length equal to the
    number of observations.}
}

\value{
  An object of class \code{"loess.simpleboot"} (which is a list)
  containing the elements:
  \item{method}{Which method of bootstrapping was used (rows or
    residuals).}
  \item{boot.list}{A list containing values from each of the bootstrap
    samples.  Currently, only residual sum of squares and fitted values
    are stored.}
  \item{orig.loess}{The original loess fit.}
  \item{new.xpts}{The locations where predictions were made (specified
    in the original call to \code{loess.boot}).}
}

\details{
  The user can specify locations for new predictions through
  \code{new.xpts} or an evenly spaced grid will be used.  In either
  case, fitted values at each new location will be stored from each
  bootstrap sample.  These fitted values can be retrieved using either
  the \code{fitted} method or the \code{samples} function.

  Note that the \code{loess} function has many parameters for the user
  to set that can be difficult to reproduce in the bootstrap setting.
  Right now, the user can only specify the \code{span} argument to
  \code{loess} in the original fit.  
}


\author{Roger D. Peng}

\keyword{loess}

\examples{
set.seed(1234)

x <- runif(100)

## Simple sine function simulation
y <- sin(2*pi*x) + .2 * rnorm(100)
plot(x, y)  ## Sine function with noise
lo <- loess(y ~ x, span = .4)

## Bootstrap with resampling of rows
lo.b <- loess.boot(lo, R = 500)

## Plot original fit with +/- 2 std. errors
plot(lo.b)

## Plot all loess bootstrap fits
plot(lo.b, all.lines = TRUE)

## Bootstrap with resampling residuals
lo.b2 <- loess.boot(lo, R = 500, rows = FALSE)
plot(lo.b2)

}

\eof
\name{loess.simpleboot.methods}

\alias{fitted.loess.simpleboot}

\title{
  Methods for loess bootstrap.
}

\usage{
\method{fitted}{loess.simpleboot}(object, ...)
}

\description{
  Methods for \code{"loess.simpleboot"} class objects.
}

\arguments{
  \item{object}{An object of class \code{"loess.simpleboot"} as
    returned by the function \code{loess.boot}.}
  \item{...}{Other arguments passed to and from other methods.}
}

\details{
  \code{fitted} returns a n x R matrix of fitted values where n is the
  number of new locations at which predictions were made and R is the
  number of bootstrap replications used in the original loess
  bootstrap.  This is the equivalent of calling \code{samples(object,
    "fitted")}.
}

\author{Roger D. Peng}

\value{
  Nothing is returned.
}


\keyword{loess}

\eof
\name{one.boot}
\alias{one.boot}
\title{
  One sample bootstrap of a univariate statistic.
}
\usage{
one.boot(data, FUN, R, student = FALSE, M, weights = NULL, ...)
}
\description{
  \code{one.boot} is used for bootstrapping a univariate statistic for
  one sample problems.  Examples include the \code{mean}, \code{median},
  etc.
}
\arguments{
  \item{data}{The data.  This should be a vector of numbers.}
  \item{FUN}{The statistic to be bootstrapped.  This can be either a
    quoted string containing the name of a function or simply the
    function name.}
  \item{R}{The number of bootstrap replicates to use.}
  \item{student}{Should we do a studentized bootstrap?  This requires a
    double bootstrap so it might take longer.}
  \item{M}{If \code{student} is set to \code{TRUE}, then \code{M} is the
    number of internal bootstrap replications to do.}
  \item{weights}{Resampling weights; a vector of length equal to the
    number of observations.}
  \item{...}{Other (named) arguments that should be passed to \code{FUN}.}
}
\value{
  An object of class \code{"simpleboot"}, which is almost identical to the
  regular \code{"boot"} object.  For example, the \code{boot.ci}
  function can be used on this object.
}
\examples{
set.seed(20)
x <- rgamma(100, 1)
b.mean <- one.boot(x, mean, 1000)
print(b.mean)
boot.ci(b.mean)  ## No studentized interval here
hist(b.mean)

## This next line could take some time on a slow computer
b.median <- one.boot(x, median, R = 500, student = TRUE, M = 50)
boot.ci(b.median)
hist(b.median)

## Bootstrap with weights
set.seed(10)
w <- runif(100)
bw <- one.boot(x, median, 1000, weights = w)
print(bw)

## Studentized
bw.stud <- one.boot(x, median, R = 500, student = TRUE, M = 50,
                    weights = w)
boot.ci(bw.stud, type = "stud")
}

\keyword{univar}

\author{Roger D. Peng}

\eof
\name{pairs.boot}
\alias{pairs.boot}

\title{
  Two sample bootstrap.
}

\usage{
pairs.boot(x, y = NULL, FUN, R, student = FALSE, M, weights = NULL, ...)
}

\description{
  \code{pairs.boot} is used to bootstrap a statistic which operates on
  two samples and returns a single value.  An example of such a
  statistic is the correlation coefficient (i.e. \code{cor}).
  Resampling is done pairwise, so \code{x} and \code{y} must have the
  same length (and be ordered correctly).  One can alternatively pass a
  two-column matrix to \code{x}.
}

\arguments{
  \item{x}{Either a vector of numbers representing the first sample or a
    two column matrix containing both samples.}
  \item{y}{If NULL it is assumed that \code{x} is a two-column matrix.
    Otherwise, \code{y} is the second sample.}
  \item{FUN}{The statistic to bootstrap.  If \code{x} and \code{y} are
    separate vectors then \code{FUN} should operate on separate
    vectors.  Similarly, if \code{x} is a matrix, then \code{FUN} should
    operate on two-column matrices.  \code{FUN} can be either a quoted
    string or a function name.}
  \item{R}{The number of bootstrap replicates.}
  \item{student}{Should we do a studentized bootstrap?  This requires a
    double bootstrap so it might take longer.}
  \item{M}{If \code{student} is set to \code{TRUE}, then \code{M} is the
    number of internal bootstrap replications to do.}
  \item{weights}{Resampling weights.}
  \item{...}{Other (named) arguments that should be passed to \code{FUN}.}
}

\value{
  An object of class \code{"simpleboot"}, which is almost identical to the
  regular \code{"boot"} object.  For example, the \code{boot.ci}
  function can be used on this object.
}

\examples{
set.seed(1)
x <- rnorm(100)
y <- 2 * x + rnorm(100)
boot.cor <- pairs.boot(x, y, FUN = cor, R = 1000)
boot.ci(boot.cor)

## With weighting
set.seed(20)
w <- (100:1)^2
bw <- pairs.boot(x, y, FUN = cor, R = 5000, weights = w)
boot.ci(bw, type = c("norm", "basic", "perc"))
}

\keyword{univar}

\author{Roger D. Peng}

\eof
\name{perc}
\alias{perc}
\alias{perc.lm}

\title{
  Extract percentiles from a bootstrap sampling distribution.
}

\usage{
perc(boot.out, p = c(0.025, 0.975))
perc.lm(lm.boot.obj, p)
}

\description{
  \code{perc} can be used to extract percentiles from the sampling
  distribution of a statistic.
}

\arguments{
  \item{boot.out}{Output from either \code{one.boot}, \code{two.boot},
    or \code{pairs.boot}.}
  \item{p}{numeric vector with values in [0, 1].}
  \item{lm.boot.obj}{An object of class \code{"lm.simpleboot"},
    returned from \code{lm.boot}.}
}

\details{
  \code{perc} automatically calls \code{perc.lm} if \code{boot.out} is
  of the class \code{"lm.simpleboot"} so there is no need to use
  \code{perc.lm} separately.
}


\value{
  For bootstraps which are not linear model bootstraps, \code{perc}
  returns a vector of percentiles of length
  \code{length(p)}.  Linear interpolation of percentiles is done if
  necessary.  \code{perc.lm} returns a matrix of percentiles
  of each of the model coefficients.  For example, if there are k model
  coefficients, the \code{perc.lm} returns a \code{length(p)} by k matrix.
}

\examples{
x <- rnorm(100)
b <- one.boot(x, median, R = 1000)
perc(b, c(.90, .95, .99))
}

\keyword{univar}
  
\author{Roger D. Peng}

\eof
\name{plot.lm.simpleboot}

\alias{plot.lm.simpleboot}

\title{
  Plot method for linear model bootstraps.
}

\usage{
\method{plot}{lm.simpleboot}(x, add = FALSE, ...)
}

\description{
  Plot regression lines with bootstrap standard errors.  This method
  only works for 2-D regression fits.
}

\arguments{
  \item{x}{An object of class \code{"lm.simpleboot"} returned by
    \code{lm.boot}.}
  \item{add}{Switch indicating whether the regression line should be
    added to the current plot.}
  \item{...}{Additional arguments passed down to \code{plot}.  Ignored
    if \code{add = TRUE}.}
}

\details{
  This function plots the data and the original regression line fit
  along with +/- 2 bootstrap standard errors at locations specified by
  the \code{new.xpts} argument to \code{lm.boot} (or on an evenly spaced
  grid).
}


\author{Roger D. Peng}

\value{
  Nothing is returned.
}

\examples{
## None right now
}

\keyword{regression}

\eof
\name{plot.loess.simpleboot}

\alias{plot.loess.simpleboot}

\title{
  Plot method for loess bootstraps.
}

\usage{
\method{plot}{loess.simpleboot}(x, add = FALSE, all.lines = FALSE, ...)
}

\description{
  Plot loess lines with bootstrap standard errors.
}

\arguments{
  \item{x}{An object of class \code{"loess.simpleboot"} as
    returned by the function \code{loess.boot}.}
  \item{add}{Should the loess line be plotted over the current plot?}
  \item{all.lines}{Should we plot each of the individual loess lines
    from the bootstrap samples?}
  \item{...}{Other arguments passed to \code{plot}.}
}

\details{
  \code{plot} constructs (and plots) the original loess fit and +/- 2
  bootstrap standard errors at locations specified in the \code{new.xpts}
  in \code{loess.boot} (or on an evenly spaced grid).
}

\author{Roger D. Peng}

\value{
  Nothing is returned.
}

\examples{
## See the help page for `loess.boot' for an example.
}

\keyword{loess}

\eof
\name{samples}
\alias{samples}

\title{
  Extract sampling distributions from bootstrapped linear/loess models.
}
\usage{
samples(object, name = c("fitted", "coef", "rsquare", "rss"))
}

\description{
  Extract sampling distributions of various entities from either a
  linear model or a loess bootstrap.  Entities for linear models are
  currently, model coefficients, residual sum of squares, R-square, and
  fitted values (given a set of X values in the original bootstrap).
  For loess, one can extract residual sum of squares and fitted values.
}

\arguments{
  \item{object}{The output from either \code{lm.boot} or
    \code{loess.boot}.}
  \item{name}{The name of the entity to extract.  The default is fitted
    values.}
}

\value{
  Either a vector or matrix depending on the entity extracted.  For
  example, when extracting the sampling distributions for linear model
  coefficents, the return value is p x R matrix where p is the number of
  coefficients and R is the number of bootstrap replicates.
}

\examples{
data(airquality)
attach(airquality)
lmodel <- lm(Ozone ~ Solar.R + Wind)
lboot <- lm.boot(lmodel, R = 500)

## Get sampling distributions for coefficients
s <- samples(lboot, "coef")

## Histogram for the intercept
hist(s[1,])

}

\keyword{utilities}

\author{Roger D. Peng}

\eof
\name{simpleboot-internal}
\alias{simpleboot-internal}
\alias{lm.boot.resample}
\alias{lo.boot.resample}
\alias{model.frame.lm.simpleboot}

\title{Internal simpleboot functions}
\description{
  Internal \code{simpleboot} functions.
}
\usage{
lm.boot.resample(lm.object, R, rows, new.xpts, weights)
lo.boot.resample(lo.object, R, rows, new.xpts, weights)
model.frame.lm.simpleboot(formula, ...)
}
\details{
  This functions are not to be called by the user.
}
\keyword{internal}

\author{Roger D. Peng}

\eof
\name{two.boot}
\alias{two.boot}

\title{
  Two sample bootstrap of differences between univariate statistics.
}

\usage{
two.boot(sample1, sample2, FUN, R, student = FALSE, M, weights = NULL, ...)
}

\description{
  \code{two.boot} is used to bootstrap the difference between various
  univariate statistics.  An example is the difference of means.
  Bootstrapping is
  done by independently resampling from \code{sample1} and \code{sample2}.
}

\arguments{
  \item{sample1}{First sample; a vector of numbers.}
  \item{sample2}{Second sample; a vector of numbers.}
  \item{FUN}{The statistic which is applied to each sample.  This can be
    a quoted string or a function name.}
  \item{R}{Number of bootstrap replicates.}
  \item{student}{Should we do a studentized bootstrap?  This requires a
    double bootstrap so it might take longer.}
  \item{M}{If \code{student} is set to \code{TRUE}, then \code{M} is the
    number of internal bootstrap replications to do.}
  \item{weights}{Resampling weights; a list with two components.  The
    first component of the list is a vector of weights for
    \code{sample1} and the second component of the list is a vector of
    weights for \code{sample2}.}
  \item{...}{Other (named) arguments that should be passed to
    \code{FUN}.}
}

\details{
  The differences are always taken as \code{FUN(sample1) -
    FUN(sample2)}.  If you want the difference to be reversed you need
  to reverse the order of the arguments \code{sample1} and
  \code{sample2}.
}
\value{
  An object of class \code{"simpleboot"}, which is almost identical to the
  regular \code{"boot"} object.  For example, the \code{boot.ci}
  function can be used on this object.
}

\examples{
set.seed(50)
x <- rnorm(100, 1)  ## Mean 1 normals
y <- rnorm(100, 0)  ## Mean 0 normals
b <- two.boot(x, y, median, R = 1000)
boot.ci(b)  ## No studentized confidence intervals
hist(b)  ## Histogram of the bootstrap replicates

b <- two.boot(x, y, quantile, R = 1000, probs = .75)

## With weighting

## Here all members of the first group has equal weighting
## but members of the the second have unequal weighting
w <- list(rep(1, 100), 100:1)
bw <- two.boot(x, y, median, R = 1000, weights = w)
boot.ci(b)

## Studentized
bstud <- two.boot(x, y, median, R = 500, student = TRUE, M = 50)
boot.ci(bstud, type = "stud")

## Studentized with weights
bwstud <- two.boot(x, y, median, R = 500, student = TRUE, M = 50,
                   weights = w)
boot.ci(bstud, type = "stud")
}

\keyword{univar}

\author{Roger D. Peng}

\eof
